from etgen.html import HtmlNamespace



E = HtmlNamespace(None, set("""
ObjectListItem
SideNavigation
item
NavigationList
NavigationListitem
fixedItem
mvc:View
m:Button

a
abbr
acronym
address
alt
applet
area
b
base
basefont
bdo
big
blockquote
body
br
button
caption
center
cite
code
col
colgroup
dd
del
dfn
dir
div
dl
dt
em
fieldset
font
form
frame
frameset
h1
h2
h3
h4
h5
h6
head
height
hr
html
i
iframe
img
input
ins
isindex
kbd
label
legend
li
link
map
menu
meta
name
noframes
noscript
object
ol
optgroup
option
p
param
pre
q
s
samp
script
select
small
span
strike
strong
style
sub
sup
table
tbody
td
textarea
tfoot
th
thead
title
tr
tt
u
ul
var
class
id
bgcolor
cellspacing
width
align
valign
href
type
rel
target
value
onclick
src
rows
data-toggle
tabindex
placeholder
""".split()))
